import click
import subprocess
from one.utils.config import get_config_value


class App:
    pass


class AppRegistry:
    def __init__(self):
        self.name = get_config_value('app.name')
        self.image_name = get_config_value('app.docker.image_name', get_config_value('app.name'))
        self.dockerfile = get_config_value('app.docker.file', 'Dockerfile')
        self.build_cmd_args = get_config_value('app.docker.build_cmd_args', '')

    def get_image_tag(self, build_version):
        return "%s:%s" % (self.image_name, build_version)

    def docker_build_raw(self, image):
        command = ['docker', 'build', '-t', image, '-f', self.dockerfile] + self.build_cmd_args.split(' ') + ['.']
        click.echo(" ".join(command))
        subprocess.call(list(filter(None, command)))

    def docker_push_raw(self, image):
        command = ['docker', 'push', image]
        click.echo(" ".join(command))
        subprocess.call(list(filter(None, command)))



def parse_env(env):
    lines = env.split("\n")
    environment = {}

    for line in lines:
        if line.strip() == "" or line[0] == '#':
            continue

        parse_line = line.strip().split('=', 1)
        if len(parse_line) == 2:
            k, v = parse_line
            environment[k] = v

    return environment



import configparser
from one.__init__ import CLI_ROOT


def get_config(file: str):
    config = configparser.ConfigParser()
    config.read(CLI_ROOT + file)

    return config


def get_credentials_file():
    credentials_file = get_config('/credentials')
    return credentials_file


def get_config_file():
    config_file = get_config('/config')
    return config_file


def get_idp_file():
    idp_file = get_config('/idp')
    return idp_file


def write_config(config, file: str):
    with open(CLI_ROOT + file, 'w+') as configfile:
        config.write(configfile)


def create_secrets(credential, path):
    file = ''
    for key, value in credential.items():
        file += '%s=%s\n' % (key, value)

    with open(path, 'w+') as f:
        f.write(file)
        f.close()



import click


def print_progress_bar(iteration, total, prefix='', suffix='', decimals=1, length=100, fill='=', printEnd='\r'):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
        printEnd    - Optional  : end character (e.g. "\r", "\r\n") (Str)
    """
    percent = ('{0:.' + str(decimals) + 'f}').format(100 * (iteration / float(total)))
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + '-' * (length - filledLength)
    print('\r%s [%s] %s%% %s' % (prefix, bar, percent, suffix), end=printEnd)
    # Print New Line on Complete
    if iteration == total:
        click.echo()



import click
from one.docker.container import Container
from one.docker.image import Image
from one.utils.environment.aws import EnvironmentAws
from one.utils.config import get_config_value, get_workspace_value
from one.utils.app.common import app_deploy_factory, app_registry_factory

image = Image()
container = Container()
environment = EnvironmentAws()

ECS_DEPLOY_IMAGE = image.get_image('ecs_deploy')
AWS_IMAGE = image.get_image('aws')


@click.group(help='Group of app commands wrapped inside docker.')
def app():
    pass


@app.command(name='docker-build', help='Build docker image for deployment.')
@click.option('--build-version', default='latest', help='Build version, used as tag for docker image.')
def docker_build(build_version):
    app_registry = app_registry_factory(get_config_value('app.docker.registry_type', 'ecr'))
    app_registry.docker_build(build_version)


@app.command(name='docker-login', help='Login into docker registry.')
def docker_login():
    app_registry = app_registry_factory(get_config_value('app.docker.registry_type', 'ecr'))
    app_registry.docker_login(environment)


@app.command(name='docker-push', help='Push image to docker registry.')
@click.option('--build-version', default='latest', help='Build version, used as tag for docker image.')
def docker_push(build_version):
    app_registry = app_registry_factory(get_config_value('app.docker.registry_type', 'ecr'))
    app_registry.docker_push(build_version)


@app.command(name='deploy-ecs', help='Deploy application to ECS.')
@click.option('-w', '--workspace', required=True, help='Workspace to deploy.')
@click.option('--build-version', default='latest', help='Build version to deploy (default: latest).')
def deploy_ecs(workspace, build_version):
    app_deploy = app_deploy_factory(get_workspace_value(workspace, 'type', 'ecs'))
    app_registry = app_registry_factory(get_config_value('app.docker.registry_type', 'ecr'))

    environment.build(workspace)
    image_name = app_registry.get_image_name(build_version)

    click.echo('Deploying %s to %s' % (image_name, workspace))

    app_deploy.deploy(environment, workspace, image_name)


@app.command(name='deploy-static', help='Deploy static application to S3 bucket.')
@click.option('-w', '--workspace', default=None, help='Workspace to deploy.')
def deploy_static(workspace):
    app_deploy = app_deploy_factory('static')

    environments = environment.build(workspace).get_env()

    click.echo('Deploying static app to %s' % (environments.get('WORKSPACE', 'default')))

    app_deploy.deploy(environments)



from one.docker.image import AZURE_AUTH_IMAGE, GSUITE_AUTH_IMAGE, TERRAFORM_IMAGE


CREATION_QUESTION = [
    {
        'type': 'input',
        'name': 'create',
        'message': 'Do you want to create workspaces now? [Y/n]',
        'default': 'Y'
    }
]


IMAGE_QUESTIONS = [
    {
        'type': 'input',
        'name': 'terraform',
        'default': TERRAFORM_IMAGE,
        'message': 'Terraform docker image:',
        'validate': lambda text: len(text) >= 4 or 'Must be at least 4 character.'
    },
    {
        'type': 'input',
        'name': 'gsuite',
        'default': GSUITE_AUTH_IMAGE,
        'message': 'G Suite docker image:',
        'validate': lambda text: len(text) >= 4 or 'Must be at least 4 character.'
    },
    {
        'type': 'input',
        'name': 'azure',
        'default': AZURE_AUTH_IMAGE,
        'message': 'Azure docker image:',
        'validate': lambda text: len(text) >= 4 or 'Must be at least 4 character.'
    },
]


WORKSPACE_QUESTIONS = [
    {
        'type': 'input',
        'name': 'AWS_ACCOUNT_ID',
        'message': 'What\'s your AWS_ACCOUNT_ID credential:',
        'validate': lambda text: len(text) >= 1 or 'Must be at least 1 character.'
    },
    {
        'type': 'input',
        'name': 'AWS_ROLE',
        'message': 'What\'s your AWS_ROLE credential:',
        'validate': lambda text: len(text) >= 1 or 'Must be at least 1 character.'
    },
    {
        'type': 'input',
        'name': 'WORKSPACE',
        'message': 'What\'s your WORKSPACE credential:',
        'validate': lambda text: len(text) >= 1 or 'Must be at least 1 character.'
    },
    {
        'type': 'input',
        'name': 'assume_role',
        'default': 'n',
        'message': 'Do you want to this workspace to assume role? [Y/n]'
    },
    {
        'type': 'input',
        'name': 'new_workspace',
        'default': 'Y',
        'message': 'Do you want to create another workspace? [Y/n]'
    }
]



from one.utils.environment.common import get_credentials_file, get_config_file

DEFAULT_REGION = 'ap-southeast-2'

AWS_ACCESS_KEY_QUESTIONS = [
    {
        'type': 'input',
        'name': 'PROFILE',
        'default': 'default',
        'message': 'What\'s your profile name:',
        'validate': lambda text: len(text) >= 4 or 'Must be at least 4 characters.'
    },
    {
        'type': 'input',
        'name': 'AWS_ACCESS_KEY_ID',
        'message': 'What\'s your AWS_ACCESS_KEY_ID:',
        'validate': lambda text: len(text) >= 4 or 'Must be at least 4 characters.'
    },
    {
        'type': 'input',
        'name': 'AWS_SECRET_ACCESS_KEY',
        'message': 'What\'s your AWS_SECRET_ACCESS_KEY:',
        'validate': lambda text: len(text) >= 4 or 'Must be at least 4 characters.'
    },
    {
        'type': 'input',
        'name': 'REGION',
        'default': DEFAULT_REGION,
        'message': 'What\'s your default REGION:',
        'validate': lambda text: len(text) >= 4 or 'Must be at least 4 characters.'
    }
]


def get_sso_profile_questions():
    aws_sso_auth_profiles = get_config_file().sections()

    if not aws_sso_auth_profiles:
        raise SystemExit

    aws_sso_auth_profiles = list(
                                map(
                                    lambda x: x.replace('profile ', ''),
                                    aws_sso_auth_profiles
                                )
                            )

    AWS_SSO_PROFILE_QUESTIONS = [
        {
            'type': 'list',
            'message': 'Select AWS SSO profile:',
            'name': 'profile',
            'choices': aws_sso_auth_profiles
        }
    ]

    return AWS_SSO_PROFILE_QUESTIONS


def get_iam_profile_questions():
    aws_iam_auth_profiles = get_credentials_file().sections()

    if not aws_iam_auth_profiles:
        raise SystemExit

    AWS_IAM_PROFILE_QUESTIONS = [
        {
            'type': 'list',
            'message': 'Select AWS IAM profile:',
            'name': 'profile',
            'choices': aws_iam_auth_profiles
        }
    ]

    return AWS_IAM_PROFILE_QUESTIONS



import click
from PyInquirer import prompt
import yaml
from one.utils.prompt import style
from one.__init__ import CONFIG_FILE
from one.prompt.init import CREATION_QUESTION, IMAGE_QUESTIONS, WORKSPACE_QUESTIONS


@click.command(help='Create config file for CLI in current directory.')
def init():
    create_answer = prompt(CREATION_QUESTION, style=style)
    create_workspace = create_answer['create'].lower()
    workspaces = {}
    if create_workspace == 'y' or not create_workspace:
        image_answers = prompt(IMAGE_QUESTIONS, style=style)
        images = {
            'terraform': image_answers['terraform'],
            'gsuite': image_answers['gsuite'],
            'azure': image_answers['azure']
        }

        while True:
            workspace_answers = prompt(WORKSPACE_QUESTIONS, style=style)
            if workspace_answers['assume_role'].lower() == 'y' or not workspace_answers['assume_role']:
                assume_role = True
            else:
                assume_role = False
            workspace = {
                'aws': {
                    'role': workspace_answers['AWS_ROLE'],
                    'account_id': workspace_answers['AWS_ACCOUNT_ID'],
                    'assume_role': assume_role
                }
            }
            workspaces[workspace_answers['WORKSPACE']] = workspace
            if workspace_answers['new_workspace'].lower() == 'n':
                break
        with open(CONFIG_FILE, 'w') as file:
            content = {'images': images, 'workspaces': workspaces}
            yaml.dump(content, file)



import click
from one.utils.prompt import style
from PyInquirer import prompt
from one.utils.config import get_workspaces, get_current_workspace_value
from one.__init__ import WORKSPACE_FILE


@click.group(help='Manage workspaces.')
def workspace():
    pass


@workspace.command(name='list', help='List all workspaces.')
def list_workspaces():
    workspaces = get_workspaces()
    current_workspace = get_current_workspace_value()

    for workspace in workspaces:
        if workspace == current_workspace:
            click.echo('* ' + click.style(workspace, fg='green', bold=True))
        else:
            click.echo('  ' + click.style(workspace))


@workspace.command(help='Change environment variables to another workspace.')
@click.option('-w', '--workspace', default=None, type=str, help='Workspace name.')
def change(workspace: str):
    workspaces = get_workspaces()

    if workspace in workspaces:
        click.echo('Selected workspace: ' + click.style(workspace, fg='green', bold=True))
    else:
        if workspace:
            click.echo('Workspace ' + click.style(workspace, fg='red', bold=True) + ' not found.', err=True)

        workspaces_obj = []
        for workspace in workspaces:
            workspaces_obj.append({'name': workspace})

        questions = [
            {
                'type': 'list',
                'message': 'Select workspace',
                'name': 'workspace',
                'choices': workspaces_obj
            }
        ]
        try:
            answers = prompt(questions, style=style)
        except KeyError:
            raise SystemExit
        workspace = answers.get('workspace', 'default')

    f = open(WORKSPACE_FILE, 'w')
    f.write('WORKSPACE=' + workspace + '\n')
    f.close()


@workspace.command(name='show', help='Show current workspace.')
def workspace_show():
    workspace = get_current_workspace_value() or 'Not found.'
    click.echo('Selected workspace: ' + click.style(workspace, fg='green', bold=True))



#!/usr/bin/env python3

import os
from os import path
import click
from dotenv import load_dotenv
from one.__init__ import __version__, WORKSPACE_FILE, CLI_ROOT
from one.commands.app import app
from one.commands.init import init
from one.commands.auth import auth
from one.commands.terraform import terraform
from one.commands.workspace import workspace
from one.commands.aws import aws, aws_v2
from one.commands.shell import shell
from one.utils.plugins import load_plugins
from one.utils.config import required_version_check


if not path.exists(CLI_ROOT):
    os.mkdir(CLI_ROOT)

required_version_check()

load_dotenv(dotenv_path=WORKSPACE_FILE)


@click.version_option(__version__)
@click.group()
def cli():
    """CLI to manage all stacks from DNX."""
    pass


COMMAND_DIRS = [aws, aws_v2, shell, app, init, auth, terraform, workspace]


for command in COMMAND_DIRS:
    cli.add_command(command)

load_plugins()


if __name__ == "__main__":
    cli()



import click
from one.docker.container import Container
from one.docker.image import Image
from one.utils.environment.aws import EnvironmentAws
from one.utils.terraform_modules import terraform_modules_check

image = Image()
container = Container()
environment = EnvironmentAws()
TERRAFORM_IMAGE = image.get_image('terraform')


@click.group(help='Group of terraform commands wrapped inside docker.')
def terraform():
    pass


@terraform.command(help='Run terraform init inside the docker container.')
@click.option('-w', '--workspace', default=None, type=str, help='Workspace to use.')
@click.option('-r', '--aws-role', 'aws_role', default=None, type=str, help='AWS role to use.')
def init(workspace, aws_role):
    envs = environment.build(workspace, aws_role).get_env()

    container.create(
        image=TERRAFORM_IMAGE,
        command='init',
        volumes=['.:/work'],
        environment=envs
    )

    command_create_workspace = 'workspace new %s' % (envs['WORKSPACE'])
    container.create(
        image=TERRAFORM_IMAGE,
        command=command_create_workspace,
        volumes=['.:/work'],
        environment=envs
    )

    command_select_workspace = 'workspace "select" %s' % (envs['WORKSPACE'])
    container.create(
        image=TERRAFORM_IMAGE,
        command=command_select_workspace,
        volumes=['.:/work'],
        environment=envs
    )

    terraform_modules_check()


@terraform.command(help='Run terraform plan inside the docker container.')
@click.option('-w', '--workspace', default=None, type=str, help='Workspace to use.')
@click.option('-r', '--aws-role', 'aws_role', default=None, type=str, help='AWS role to use.')
def plan(workspace, aws_role):
    envs = environment.build(workspace, aws_role).get_env()
    command = 'plan -out=.terraform-plan-' + envs['WORKSPACE']
    container.create(
        image=TERRAFORM_IMAGE,
        command=command,
        volumes=['.:/work'],
        environment=envs
    )


@terraform.command(help='Run terraform apply inside the docker container.')
@click.option('-w', '--workspace', default=None, type=str, help='Workspace to use.')
@click.option('-r', '--aws-role', 'aws_role', default=None, type=str, help='AWS role to use.')
def apply(workspace, aws_role):
    envs = environment.build(workspace, aws_role).get_env()
    command = 'apply .terraform-plan-' + envs['WORKSPACE']
    container.create(
        image=TERRAFORM_IMAGE,
        command=command,
        volumes=['.:/work'],
        environment=envs
    )


@terraform.command(help='Run shell and get inside the container with interactive mode.')
@click.option('-w', '--workspace', default=None, type=str, help='Workspace to use.')
@click.option('-r', '--aws-role', 'aws_role', default=None, type=str, help='AWS role to use.')
def shell(workspace, aws_role):
    envs = environment.build(workspace, aws_role).get_env()
    container.create(
        image=TERRAFORM_IMAGE,
        entrypoint='/bin/bash',
        volumes=['.:/work'],
        environment=envs
    )


@terraform.command(help='Run terraform destroy inside the docker container.')
@click.option('-w', '--workspace', default=None, type=str, help='Workspace to use.')
@click.option('-r', '--aws-role', 'aws_role', default=None, type=str, help='AWS role to use.')
def destroy(workspace, aws_role):
    envs = environment.build(workspace, aws_role).get_env()
    container.create(
        image=TERRAFORM_IMAGE,
        command='destroy',
        volumes=['.:/work'],
        environment=envs
    )


@terraform.command(help='Run terraform force-unlock inside the docker container.')
@click.argument('lock_id')
def force_unlock(lock_id):
    envs = environment.build().get_env()
    container.create(
        image=TERRAFORM_IMAGE,
        command='force-unlock',
        volumes=['.:/work'],
        environment=envs
    )



import click
from one.docker.client import client
from one.utils.print_progress_bar import print_progress_bar
from one.utils.config import get_config_value
from requests.exceptions import ConnectionError


GSUITE_AUTH_IMAGE = 'dnxsolutions/aws-google-auth:latest'
AZURE_AUTH_IMAGE = 'dnxsolutions/docker-aws-azure-ad:latest'
TERRAFORM_IMAGE = 'dnxsolutions/terraform:0.13.0-dnx1'
AWS_IMAGE = 'dnxsolutions/aws:1.18.44-dnx2'
AWS_V2_IMAGE = 'dnxsolutions/aws:2.0.37-dnx1'
SHELL_IMAGE = 'dnxsolutions/shell:latest'
ECS_DEPLOY_IMAGE = 'dnxsolutions/ecs-deploy:1.2.0'


class Image:

    def __init__(self):
        pass

    def get_images(self):
        images = {'terraform': TERRAFORM_IMAGE,
                  'gsuite': GSUITE_AUTH_IMAGE,
                  'azure': AZURE_AUTH_IMAGE,
                  'aws': AWS_IMAGE,
                  'aws_v2': AWS_V2_IMAGE,
                  'shell': SHELL_IMAGE,
                  'ecs_deploy': ECS_DEPLOY_IMAGE}

        for key, value in images.items():
            images.update({
                key: get_config_value('images.' + key, value)
            })
        return images

    def get_image(self, key):
        images = self.get_images()
        return images[key]

    def check_image(self, image):
        try:
            docker_image = client.images(name=image, all=True)
        except ConnectionError:
            click.echo('Error: Make sure Docker is running (requests.exceptions.ConnectionError)')
            raise SystemExit

        if not docker_image:
            self.pull(image)

    def pull(self, image):
        for line in client.pull(image, stream=True, decode=True):
            if 'progressDetail' in line:
                progress_detail = line['progressDetail']
                if len(progress_detail) != 0:
                    current = progress_detail['current']
                    total = progress_detail['total']
                    prefix = line['id']
                    suffix = line['status']
                    print_progress_bar(iteration=current,
                                       total=total,
                                       prefix=prefix,
                                       suffix=suffix,
                                       fill='=')



import click
from one.utils.app.registry.ecr import AppRegistryEcr
from one.utils.app.deploy.ecs import AppDeployEcs
from one.utils.app.deploy.static import AppDeployStatic


def app_deploy_factory(type):
    if type == 'ecs':
        return AppDeployEcs()
    elif type == 'static':
        return AppDeployStatic()
    else:
        click.echo('Type not implemented. Valid values: ecs, static.')
        raise SystemExit


def app_registry_factory(registry_type):
    if registry_type == 'ecr':
        return AppRegistryEcr()
    else:
        click.echo('Docker registry_type not implemented. Valid values: ecr')
        raise SystemExit



prompt_providers = ['Google G Suite SSO', 'Microsoft Azure SSO', 'AWS SSO', 'AWS IAM user']

PROVIDER_QUESTIONS = [
    {
        'type': 'list',
        'message': 'Select authentication method:',
        'name': 'provider',
        'choices': prompt_providers
    }
]

GSUITE_QUESTIONS = [
    {
        'type': 'input',
        'name': 'GOOGLE_IDP_ID',
        'message': 'What\'s your GOOGLE_IDP_ID credential:',
        'validate': lambda text: len(text) >= 4 or 'Must be at least 4 characters.'
    },
    {
        'type': 'input',
        'name': 'GOOGLE_SP_ID',
        'message': 'What\'s your GOOGLE_SP_ID credential:',
        'validate': lambda text: len(text) >= 4 or 'Must be at least 4 characters.'
    }
]

AZURE_QUESTIONS = [
    {
        'type': 'input',
        'name': 'AZURE_TENANT_ID',
        'message': 'What\'s your AZURE_TENANT_ID credential:',
        'validate': lambda text: len(text) >= 4 or 'Must be at least 4 characters.'
    },
    {
        'type': 'input',
        'name': 'AZURE_APP_ID_URI',
        'message': 'What\'s your AZURE_APP_ID_URI credential:',
        'validate': lambda text: len(text) >= 4 or 'Must be at least 4 characters.'
    }
]



import click
import os
from one.utils.config import get_workspace_value, get_config_value
from one.docker.container import Container
from one.docker.image import Image
from one.utils.app import App

AWS_IMAGE = Image().get_image('aws')
container = Container()


class AppDeployStatic(App):
    def __init__(self):
        super().__init__()

    def deploy(self, environments):
        workspace = environments.get('WORKSPACE', 'default')

        env_deploy = {
            'AWS_DEFAULT_REGION': get_workspace_value(workspace, 'aws.region')
        }
        environments.update(env_deploy)

        s3_bucket_name = get_config_value('app.s3_bucket', '') or get_workspace_value(workspace, 'app.s3_bucket')
        distribution_id = get_config_value('app.distribution_id', '') or get_workspace_value(workspace, 'app.distribution_id')
        src_dir = get_config_value('app.src', '') or get_workspace_value(workspace, 'app.src')

        if not os.path.isdir(src_dir):
            click.echo('Source folder not found (%s)' % src_dir)
            raise SystemExit

        command_s3_sync = """s3 sync %s s3://%s --delete --cache-control \
                             max-age=31536000 --acl public-read""" % (src_dir, s3_bucket_name)
        container.create(
            command=command_s3_sync,
            image=AWS_IMAGE,
            environment=environments,
            volumes=['.:/work']
        )

        if distribution_id:
            command_cloudfront = 'cloudfront create-invalidation --distribution-id %s --paths "/*"' % (distribution_id)

            container.create(
                command=command_cloudfront,
                image=AWS_IMAGE,
                environment=environments,
                volumes=['.:/work']
            )



import click
from one.docker.container import Container
from one.docker.image import Image
from one.utils.environment.aws import EnvironmentAws


container = Container()
image = Image()
environment = EnvironmentAws()
AWS_IMAGE = image.get_image('aws')
AWS_V2_IMAGE = image.get_image('aws_v2')


@click.command(help='AWS CLI alias.')
@click.argument('args', nargs=-1)
@click.option('-w', '--workspace', default=None, type=str, help='Workspace to use.')
@click.option('-r', '--aws-role', 'aws_role', default=None, type=str, help='AWS role to use.')
def aws(args, workspace, aws_role):
    envs = environment.build(workspace, aws_role).get_env()
    command = ''
    for arg in args:
        command += '%s ' % (arg)
    container.create(
        image=AWS_IMAGE,
        command=command,
        volumes=['.:/work'],
        environment=envs
    )


@click.command(help='AWS v2 CLI alias.')
@click.argument('args', nargs=-1)
@click.option('-w', '--workspace', default=None, type=str, help='Workspace to use.')
@click.option('-r', '--aws-role', 'aws_role', default=None, type=str, help='AWS role to use.')
def aws_v2(args, workspace, aws_role):
    envs = environment.build(workspace, aws_role).get_env()
    command = ''
    for arg in args:
        command += '%s ' % (arg)
    container.create(
        image=AWS_V2_IMAGE,
        command=command,
        volumes=['.:/work'],
        environment=envs
    )



import click
import docker.utils
from os import path, getenv
from one.utils.config import get_workspace_value
from one.utils.parse_env import parse_env
from one.docker.container import Container
from one.docker.image import Image
from one.utils.environment import Environment
from one.__init__ import CLI_ROOT


class EnvironmentAws(Environment):
    def __init__(self):
        super().__init__()
        self.env_auth = {}
        self.env_assume = {}
        self.env_workspace = {}
        self.workspace = ''

    def build(self, workspace=None, aws_role=None, aws_account_id=None, aws_assume_role=None):
        if path.exists(CLI_ROOT + '/secrets'):
            self.env_auth = docker.utils.parse_env_file(CLI_ROOT + '/secrets')
        else:
            click.echo('Please login before proceeding')
            raise SystemExit

        self.workspace = workspace or getenv('WORKSPACE') or 'default'
        click.echo('Setting workspace to %s' % (self.workspace))

        aws_account_id = aws_account_id or get_workspace_value(self.workspace, 'aws.account_id')
        aws_role = aws_role or get_workspace_value(self.workspace, 'aws.role')
        aws_assume_role = aws_assume_role or get_workspace_value(self.workspace, 'aws.assume_role', 'false')

        self.env_workspace = {
            'TF_VAR_aws_role': aws_role,
            'TF_VAR_aws_account_id': aws_account_id,
            'WORKSPACE': self.workspace
        }

        if aws_assume_role.lower() == 'true':
            self.aws_assume_role(aws_role=aws_role, aws_account_id=aws_account_id)

        return self

    def aws_assume_role(self, aws_role, aws_account_id):
        click.echo('Assuming role %s at %s' % (aws_role, aws_account_id))
        container = Container()
        image = Image()

        AWS_IMAGE = image.get_image('aws')
        envs = {
            'AWS_ROLE': aws_role,
            'AWS_ACCOUNT_ID': aws_account_id,
        }
        envs.update(self.env_auth)

        command = 'assume-role.sh'
        output = container.create(
            image=AWS_IMAGE,
            entrypoint='/bin/bash -c',
            command=command,
            volumes=['.:/work'],
            environment=envs,
            tty=False,
            stdin_open=False
        )

        self.env_assume = parse_env('\n'.join(output.splitlines()))
        return self.env_assume

    def get_env(self):
        return {**self.env_auth, **self.env_assume, **self.env_workspace}



import click
from one.docker.container import Container
from one.docker.image import Image
from one.utils.environment.aws import EnvironmentAws


container = Container()
image = Image()
environment = EnvironmentAws()
SHELL_IMAGE = image.get_image('shell')


@click.command(help='Shell container with awscli and terraform pre-installed.')
@click.argument('args', nargs=-1)
@click.option('-i', '--image', default=SHELL_IMAGE, type=str, help='Docker image to use.')
@click.option('-p', '--port', default=(), type=str, help='Ports to expose from the container.', multiple=True)
def shell(args, image, port):
    envs = environment.build().get_env()
    command = ''

    for arg in args:
        command += '%s ' % (arg)

    ports = list(port)

    container.create(
        image=image,
        command=command,
        ports=ports,
        entrypoint='',
        volumes=['.:/work'],
        environment=envs
    )



from PyInquirer import style_from_dict, Token


style = style_from_dict({
    Token.Separator: '#50A8E2',
    Token.QuestionMark: '#673ab7 bold',
    Token.Selected: '#50A8E2',  # default
    Token.Pointer: '#673ab7 bold',
    Token.Instruction: '',  # default
    Token.Answer: '#f44336 bold',
    Token.Question: '',
})



import click
import os
import yaml
import re
from packaging.version import parse
from os import path
from one.__init__ import CONFIG_FILE, __version__
from one.utils.operators import OPERATORS


if not path.exists(CONFIG_FILE):
    click.echo(
        click.style('WARN ', fg='yellow') +
        'No config file in current directory.\n'
    )


def required_version_check():
    versions = get_config_value('required_version', '')
    if versions:
        required_versions = re.split(r'[,]\s*', versions)
        for version in required_versions:
            try:
                operator = OPERATORS[version.split(' ', 1)[0]]
                required_version = parse(version.split(' ')[1])
                if not operator(parse(__version__), required_version):
                    click.echo(
                        click.style('ERROR ', fg='red') +
                        'Config file requires one-cli version %s and your current version is %s.\n' % (version, __version__))
                    raise SystemExit
            except KeyError:
                click.echo(
                    click.style('ERROR ', fg='red') +
                    'Wrong configuration for required_version in one.yaml.\n')
                raise SystemExit


def get_config_value(key, default=None):
    value = default
    if path.exists(CONFIG_FILE):
        with open(CONFIG_FILE) as file:
            docs = yaml.load(file, Loader=yaml.BaseLoader)
            for key_path in key.split('.'):
                if key_path not in docs:
                    break
                if isinstance(docs[key_path], str):
                    value = docs[key_path]
                docs = docs[key_path]

        if value is None:
            click.echo('Required parameter: %s' % key)
            raise SystemExit

    return value


def get_workspace_value(workspace_name, variable, default=None):
    value = default
    if path.exists(CONFIG_FILE):
        with open(CONFIG_FILE) as file:
            docs = yaml.load(file, Loader=yaml.BaseLoader)
            if workspace_name not in docs['workspaces']:
                if workspace_name is None:
                    click.echo('Please set workspace before continuing.')
                else:
                    click.echo('Workspace %s not found.' % (workspace_name))
                raise SystemExit

            layer = docs['workspaces'][workspace_name]
            keys = variable.split('.')

            for key_path in keys:
                if key_path in layer:
                    if key_path == keys[-1]:  # Last key
                        value = layer[key_path]
                    else:
                        layer = layer[key_path]
                else:
                    break

            if not value:
                click.echo('Missing required parameter in config: workspaces.%s.%s.' % (workspace_name, variable))
                raise SystemExit

        file.close()

    return str(value)


def get_current_workspace_value(default=None):
    return os.getenv('WORKSPACE') or 'default'


def get_workspaces():
    workspaces = []
    if path.exists(CONFIG_FILE):
        with open(CONFIG_FILE) as file:
            docs = yaml.load(file, Loader=yaml.BaseLoader)
            for workspace_key in docs['workspaces'].keys():
                workspaces.append(workspace_key)
        file.close()

    return workspaces



import dockerpty
from one.docker.client import client
from one.docker.image import Image
import os


class Container:

    def __init__(self):
        pass

    def create(self, image='', command=None, entrypoint=None, volumes=[], ports=[],
               working_dir='/work', stdin_open=True, tty=True, environment=''):

        Image().check_image(image)
        host_config = None

        container_volumes = []
        binds = []
        for volume in volumes:
            volume_parts = volume.split(':')
            if volume_parts[0][0] == '.':
                volume_parts[0] = os.getcwd() + volume_parts[0][1:]
            container_volumes.append(volume_parts[1])
            binds.append(':'.join(volume_parts))

        port_bindings = {}
        for port in ports:
            port_bindings[port] = port

        host_config = client.create_host_config(
            binds=binds,
            port_bindings=port_bindings
        )

        container = client.create_container(image,
                                            command=command,
                                            entrypoint=entrypoint,
                                            stdin_open=stdin_open,
                                            tty=tty,
                                            ports=ports,
                                            environment=environment,
                                            working_dir=working_dir,
                                            volumes=container_volumes,
                                            host_config=host_config)

        if tty:
            dockerpty.start(client, container)
        else:
            client.start(container=container.get('Id'))
            client.wait(container=container.get('Id'))

        logs = client.logs(container['Id'])
        client.remove_container(container)

        return logs.decode('utf8')



import click
from PyInquirer import prompt
from one.utils.environment.common import get_credentials_file, get_config_file, get_idp_file, write_config
from one.utils.prompt import style
from one.docker.image import Image
from one.docker.container import Container
from one.__init__ import CLI_ROOT
from one.prompt.idp import PROVIDER_QUESTIONS, GSUITE_QUESTIONS, AZURE_QUESTIONS
from one.prompt.auth import AWS_ACCESS_KEY_QUESTIONS

image = Image()
container = Container()


def configure_idp():
    provider_answer = prompt(PROVIDER_QUESTIONS, style=style)
    if not provider_answer:
        raise SystemExit

    if provider_answer['provider'] == 'Google G Suite SSO':
        configure_gsuite()
    elif provider_answer['provider'] == 'Microsoft Azure SSO':
        configure_azure()
    elif provider_answer['provider'] == 'AWS SSO':
        configure_aws_sso()
    elif provider_answer['provider'] == 'AWS IAM user':
        configure_iam_user()
    else:
        raise SystemExit


def configure_gsuite():
    answers = prompt(GSUITE_QUESTIONS, style=style)
    if not bool(answers):
        raise SystemExit
    idp_file = get_idp_file()
    idp_file['gsuite'] = {
        'google_idp_id': answers['GOOGLE_IDP_ID'],
        'google_sp_id': answers['GOOGLE_SP_ID']
    }

    write_config(idp_file, '/idp')
    click.echo('\n')


def configure_azure():
    answers = prompt(AZURE_QUESTIONS, style=style)
    if not bool(answers):
        raise SystemExit

    idp_file = get_idp_file()
    idp_file['azure'] = {
        'AZURE_TENANT_ID': answers['AZURE_TENANT_ID'],
        'AZURE_APP_ID_URI': answers['AZURE_APP_ID_URI']
    }

    write_config(idp_file, '/idp')
    click.echo('\n')


def configure_aws_sso():
    auth_image = image.get_image('aws_v2')
    work_volume = CLI_ROOT + ':/work'
    env_sso = {}
    env_sso['AWS_CONFIG_FILE'] = '/work/config'

    container.create(
        image=auth_image,
        command='configure sso',
        volumes=[work_volume],
        environment=env_sso
    )
    click.echo('\n')


def configure_iam_user():
    aws_auth_answer = prompt(AWS_ACCESS_KEY_QUESTIONS, style=style)
    if not aws_auth_answer:
        raise SystemExit
    credentials_file = get_credentials_file()
    credentials_file[aws_auth_answer['PROFILE']] = {
        'AWS_ACCESS_KEY_ID': aws_auth_answer['AWS_ACCESS_KEY_ID'],
        'AWS_SECRET_ACCESS_KEY': aws_auth_answer['AWS_SECRET_ACCESS_KEY']
    }

    config_file = get_config_file()
    config_file['profile ' + aws_auth_answer['PROFILE']] = {
        'REGION': aws_auth_answer['REGION']
    }

    write_config(credentials_file, '/credentials')
    write_config(config_file, '/config')
    click.echo('\n')



import click
import subprocess
from one.utils.config import get_config_value
from one.docker.image import Image
from one.docker.container import Container
from one.utils.app import AppRegistry

AWS_IMAGE = Image().get_image('aws')
container = Container()


class AppRegistryEcr(AppRegistry):
    def __init__(self):
        super().__init__()
        self.name = get_config_value('app.name')
        self.ecr_aws_region = get_config_value('app.docker.registry_options.ecr_aws_region')
        self.ecr_aws_account_id = get_config_value('app.docker.registry_options.ecr_aws_account_id')

    def get_image_name(self, build_version):
        image_tag = super().get_image_tag(build_version)
        return "%s.dkr.ecr.%s.amazonaws.com/%s" % (self.ecr_aws_account_id, self.ecr_aws_region, image_tag)

    def docker_login(self, environment):
        aws_account_id = get_config_value('app.docker.registry_options.ecr_aws_account_id')
        aws_role = get_config_value('app.docker.registry_options.ecr_aws_role')
        aws_assume_role = get_config_value('app.docker.registry_options.ecr_aws_assume_role', 'false').lower()

        envs = environment.build(
            aws_assume_role=aws_assume_role,
            aws_role=aws_role,
            aws_account_id=aws_account_id
        ).get_env()

        docker_get_login = container.create(
                image=AWS_IMAGE,
                command="ecr get-login --no-include-email --registry-ids %s --region %s" % (
                    self.ecr_aws_account_id,
                    self.ecr_aws_region
                ),
                environment=envs,
                tty=False
        )

        if ' '.join(docker_get_login.split()[:4]) == 'Unable to locate credentials.':
            click.echo(
                click.style('ERROR: ', fg='red') +
                docker_get_login
            )
            raise SystemExit

        docker_login_command_parts = docker_get_login.strip().split(' ')

        subprocess.call(list(filter(None, docker_login_command_parts)))

        # try:
        #     output = docker_client.login(username=docker_login_username,
        # password=docker_login_password, registry=docker_login_endpoint)
        # except APIError:
        #     click.echo('Error with docker login: ', docker_get_login.strip())
        #     raise SystemExit

        click.echo("Docker login succeeded: %s.dkr.ecr.%s.amazonaws.com" % (self.ecr_aws_account_id, self.ecr_aws_region))

    def docker_build(self, build_version):
        image = self.get_image_name(build_version)
        super().docker_build_raw(image)

    def docker_push(self, build_version):
        image = self.get_image_name(build_version)
        super().docker_push_raw(image)



import glob
import importlib
import sys
import os
import click
import json
import tarfile
import shutil
import yaml
from os.path import join
from os import path
import urllib.request
from pathlib import Path
from one.utils.config import get_config_value
from one.__init__ import CONFIG_FILE


home = str(Path.home())
TEMP_FOLDER = '.one/.temp'
PLUGIN_DATA_FILE = '.one/plugins.json'


def download_plugin(extract_source, url, key):
    file_stream = urllib.request.urlopen(url)

    if url.endswith('tar.gz'):
        tar = tarfile.open(fileobj=file_stream, mode='r|gz')
        tar.extractall(TEMP_FOLDER)
        tar.close()
    elif url.endswith('tar'):
        tar = tarfile.open(fileobj=file_stream, mode='r|')
        tar.extractall(TEMP_FOLDER)
        tar.close()
    else:
        click.echo(
            click.style('ERROR ', fg='red') +
            'No supporot for this file format.\n'
        )
        raise SystemExit

    plugin_path = join(extract_source, key)
    if path.exists(plugin_path):
        shutil.rmtree(plugin_path)

    shutil.move(glob.glob('.one/.temp/*')[0], plugin_path)
    shutil.rmtree(TEMP_FOLDER)


def cleanup_plugins(extract_source, installed_plugins):
    if not path.exists(extract_source):
        return

    set_difference = set(os.listdir(extract_source)) - set(installed_plugins.keys())
    list_difference = list(set_difference)
    for directory in list_difference:
        shutil.rmtree(join(extract_source, directory))


def read_plugin_data():
    try:
        with open(PLUGIN_DATA_FILE) as json_file:
            return json.load(json_file)
    except Exception:
        return {}


def check_plugins(extract_source='.one/plugins/'):
    try:
        installed_plugins = {}
        plugin_data = read_plugin_data()

        with open(CONFIG_FILE) as file:
            docs = yaml.load(file, Loader=yaml.BaseLoader)
            for key, url in docs['plugins'].items():
                source = get_config_value('plugins.' + key + '.source')
                plugin_path = join(extract_source, key)
                if not path.exists(plugin_path) or plugin_data[key]['source'] != source:
                    click.echo('Installing plugin %s.' % key)
                    download_plugin(extract_source, source, key)
                    click.echo('Plugin %s successfully installed.\n' % key)

                plugin = {
                    'source': source,
                    'dir': join(extract_source, key)
                }
                installed_plugins[key] = plugin
        file.close()
    except KeyError:
        pass
    except AttributeError:
        click.echo(
            click.style('WARN ', fg='yellow') +
            'Plugin block declared but empty.\n'
        )
    except FileNotFoundError:
        click.echo(
            click.style('ERROR ', fg='red') +
            'Config file %s not found.\n' % CONFIG_FILE
        )
    except Exception:
        click.echo(
            click.style('ERROR ', fg='red') +
            'Unexpected error.\n'
        )
        raise
    with open(PLUGIN_DATA_FILE, 'w') as outfile:
        json.dump(installed_plugins, outfile)
    cleanup_plugins(extract_source, installed_plugins)


def load_plugins(source='.one/plugins/'):
    if not path.exists('.one'):
        os.mkdir('.one')

    check_plugins()

    if path.exists(source):
        sys.path.append('.one')
        for directory in glob.glob(join(source, '*/')):
            if path.exists(directory + '__init__.py'):
                command_path = '.'.join(directory.rsplit('/', 3)[1:])[:-1] + '.__init__'
                getattr(importlib.import_module(command_path), '__init__')()



#!/usr/bin/env python

from setuptools import setup, find_packages
from one.__init__ import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

install_requires = [
    'altgraph==0.17',
    'certifi==2020.4.5.1',
    'chardet==3.0.4',
    'click==7.1.2',
    'docker-py==1.10.6',
    'docker-pycreds==0.4.0',
    'dockerpty==0.4.1',
    'idna==2.9',
    'prompt-toolkit==1.0.14',
    'Pygments==2.6.1',
    'PyInquirer==1.0.3',
    'PyInstaller==3.6',
    'python-dotenv==0.13.0',
    'PyYAML==5.3.1',
    'regex==2020.5.7',
    'requests==2.23.0',
    'six==1.14.0',
    'urllib3==1.25.9',
    'wcwidth==0.1.9',
    'websocket-client==0.57.0',
    'packaging==20.4'
]

setup(
    name='one-cli',
    version=__version__,
    py_modules=['one'],
    include_package_data=True,
    description='Python CLI to manage stacks from DNX.',
    license="Apache License 2.0",
    url='https://github.com/DNXLabs/one-cli',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='DNX Solutions',
    author_email='contact@dnx.solutions',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ],
    packages=find_packages(),
    install_requires=install_requires,
    python_requires='>=3.6',
    entry_points='''
        [console_scripts]
        one=one.__main__:main
    ''',
)



import click
import os
from one.utils.config import get_config_value, get_workspace_value
from one.docker.container import Container
from one.docker.image import Image
from one.utils.app import App

ECS_DEPLOY_IMAGE = Image().get_image('ecs_deploy')
container = Container()


class AppDeployEcs(App):
    def __init__(self):
        super().__init__()

    def deploy(self, environment, workspace, image_name):
        env_deploy = {
            'AWS_DEFAULT_REGION': get_workspace_value(workspace, 'aws.region'),
            'APP_NAME': get_config_value('app.name'),
            'CLUSTER_NAME': get_workspace_value(workspace, 'ecs_cluster_name'),
            'CONTAINER_PORT': get_config_value('app.port'),
            'IMAGE_NAME': image_name,
        }

        ecs_task_definition_file = get_config_value('app.ecs_task_definition_file', 'task-definition.tpl.json')
        if not os.path.isfile(ecs_task_definition_file):
            click.echo('ECS task definition file not found (%s)' % ecs_task_definition_file)
            raise SystemExit

        envs = environment.get_env()

        timeout = get_workspace_value(workspace, 'deploy_timeout', '0')
        if timeout != '0':
            env_deploy['DEPLOY_TIMEOUT'] = timeout

        envs.update(env_deploy)
        container.create(
            image=ECS_DEPLOY_IMAGE,
            environment=envs,
            volumes=[
                './%s:/work/task-definition.tpl.json' % (ecs_task_definition_file)
            ]
        )



import json
import re
import click
import requests
from os import path


def terraform_modules_check():
    file_path = '.terraform/modules/modules.json'
    if not path.exists(file_path):
        raise SystemExit

    response = requests.get('https://modules.dnx.one/api.json')
    if response.status_code != 200:
        raise SystemExit
    api = response.json()

    with open('.terraform/modules/modules.json') as modules_json_file:
        data = json.load(modules_json_file)
        click.echo(
            click.style('\nInitializing DNX modules check...', bold=True)
        )
        for module in data['Modules']:

            if not module['Source']:
                continue

            split = re.split(r'[./]\s*', module['Source'])
            if len(split) >= 5 and split[4] == 'DNXLabs':
                name = re.split(r'[./]\s*', module['Source'])[5]
                version = module['Source'].split('=')[1]
                key = module['Key']
                api_version = ''
                try:
                    api_version = api['modules'][name]['tag_name']
                except KeyError:
                    click.echo(
                        click.style('ERROR ', fg='red') +
                        'Could not find module ' + name + ' at DNX modules API.'
                    )
                if api_version != version:
                    click.echo(
                        '- ' + name + '/' + key + ': ' +
                        click.style(version, fg='yellow') +
                        ' ~> ' +
                        click.style(api_version, fg='green')
                    )
                else:
                    click.echo(
                        '- ' + name + '/' + key + ': ' +
                        click.style(version, fg='green')
                    )



import click
import shutil
import configparser
import os
from os import path
from PyInquirer import prompt
from one.docker.container import Container
from one.docker.image import Image
from one.utils.prompt import style
from one.prompt.auth import get_sso_profile_questions, get_iam_profile_questions
from one.__init__ import CLI_ROOT
from one.utils.environment.common import create_secrets, get_credentials_file, get_config_file, get_idp_file
from one.utils.environment.idp import configure_idp, configure_gsuite, configure_azure, configure_aws_sso, configure_iam_user

container = Container()
image = Image()


@click.group(help='Group of auth commands.')
def auth():
    pass


@auth.command(help='Configure authentication to be used.')
def configure():
    configure_idp()


@auth.command(help='Authentication using SSO provider or AWS IAM user.')
def gsuite(auth_image=None):
    if not check_config_file(['idp']):
        configure_gsuite()
    auth_image = image.get_image('gsuite')

    with open(CLI_ROOT + '/.env', 'w') as file:
        file.write('NONE=')
        file.close()

    idp_file = get_idp_file()
    envs = {
        'GOOGLE_IDP_ID': idp_file['gsuite']['google_idp_id'],
        'GOOGLE_SP_ID': idp_file['gsuite']['google_sp_id']
    }

    credentials_volume = CLI_ROOT + ':/work'
    container.create(
        image=auth_image,
        volumes=[credentials_volume],
        environment=envs
    )

    shutil.move(CLI_ROOT + '/.env', CLI_ROOT + '/secrets')


@auth.command(help='Authentication using Azure SSO provider')
def azure():
    if not check_config_file(['idp']):
        configure_azure()
    auth_image = image.get_image('azure')
    credentials_volume = CLI_ROOT + ':/work'

    idp_file = get_idp_file()
    envs = {
        'AZURE_TENANT_ID': idp_file['azure']['azure_tenant_id'],
        'AZURE_APP_ID_URI': idp_file['azure']['azure_app_id_uri']
    }

    container.create(
        image=auth_image,
        volumes=[credentials_volume],
        environment=envs
    )

    shutil.move(CLI_ROOT + '/.env', CLI_ROOT + '/secrets')


@auth.command(help='Authentication using AWS SSO provider.')
def aws():
    if not check_config_file(['config']):
        configure_aws_sso()
    aws_sso_profile_answer = prompt(get_sso_profile_questions(), style=style)
    if not aws_sso_profile_answer:
        raise SystemExit

    auth_image = image.get_image('aws_v2')
    work_volume = CLI_ROOT + ':/work'
    envs = {
        'AWS_CONFIG_FILE': '/work/config',
        'AWS_SSO_PROFILE': aws_sso_profile_answer.get('profile', 'default')
    }

    container.create(
        image=auth_image,
        entrypoint='bash',
        command='/opt/scripts/aws-sso.sh',
        volumes=[work_volume],
        environment=envs
    )

    shutil.copy(CLI_ROOT + '/.env', CLI_ROOT + '/secrets')
    os.remove(CLI_ROOT + '/.env')


@auth.command(help='Authentication using AWS IAM user.')
def iam():
    if not check_config_file(['credentials', 'config']):
        configure_iam_user()
    aws_iam_profile_answer = prompt(get_iam_profile_questions(), style=style)

    if not aws_iam_profile_answer:
        raise SystemExit

    profile = aws_iam_profile_answer.get('profile', 'default')
    credentials_file = get_credentials_file()
    config_file = get_config_file()
    credential = {
        'AWS_ACCESS_KEY_ID': credentials_file[profile]['aws_access_key_id'],
        'AWS_SECRET_ACCESS_KEY': credentials_file[profile]['aws_secret_access_key'],
        'REGION': config_file['profile ' + profile]['region']
    }

    create_secrets(credential, CLI_ROOT + '/secrets')


def check_config_file(files: list):
    for file in files:
        file_path = '%s/%s' % (CLI_ROOT, file)
        if not path.exists(file_path):
            click.echo('\nYou do not have any %s configured, starting configuration.\n' % file)
            with open(file_path, 'w+'):
                return False

        config = configparser.ConfigParser()
        config.read(file_path)

        if not config.sections():
            click.echo('%s file is empty.\n' % file.capitalize())
            return False

    return True
